/***************************************************************************
 *   Copyright (C) 2004 by Malte Bhme                                     *
 *   malte@stammkranich.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "client.h"

#include <qpushbutton.h>
#include <qlineedit.h>
#include <qsocket.h>
#include <qfiledialog.h>
#include <qcolordialog.h>
#include <qimage.h>
#include <qcstring.h>
#include <qdatastream.h>
#include <qvariant.h>
#include <qcolordialog.h>
#include <qobject.h>
#include <qfiledialog.h>

KWBClient::KWBClient()
{
	m_socket = new QSocket(this);
	packetSize = 0;
	connect( m_socket, SIGNAL(readyRead()), SLOT(read()) );
}

void KWBClient::setSocket(const int &socket)
{
	m_socket->setSocket(socket);
}

void KWBClient::sendPacket( const QVariant &v )
{
	QByteArray ba;
	QDataStream varDs( ba, IO_WriteOnly );
	varDs << v;

	QDataStream ds( m_socket );
	ds << (Q_UINT32) ba.size();
	m_socket->writeBlock( ba.data(), ba.size() );
}

void KWBClient::sendImage()
{
	QString imageName = QFileDialog::getOpenFileName( QString::null,
	                    "Images (*.png *.xpm *.jpg)", 0 );
	QImage image( imageName );
	if ( !image.isNull() )
	{
		sendPacket( QPixmap(image) );
	}
}

void KWBClient::sendText(const QString &str)
{
	sendPacket(str);
}

void KWBClient::sendPixmap(const QPixmap &pix)
{
	sendPacket(pix);
}

void KWBClient::sendPalette()
{
	QColor col = QColorDialog::getColor( white, 0 );
	if ( col.isValid() )
	{
		sendPacket( QPalette(col,col) );
	}
}

void KWBClient::read()
{
	Q_ULONG bytesAvail = m_socket->bytesAvailable();
	for ( ;; )
	{
		if ( packetSize == 0 )
		{
			QDataStream ds( m_socket );
			if ( bytesAvail < 4 )
				return;
			ds >> packetSize;
			bytesAvail -= 4;
		}
		else
		{
			if ( bytesAvail < packetSize )
				return;
			// read the packet in a byte array to be sure that you don't
			// read too much or too less
			QByteArray ba( packetSize );
			m_socket->readBlock( ba.data(), packetSize );
			bytesAvail -= packetSize;
			packetSize = 0;

			QVariant variant;
			QDataStream ds( ba, IO_ReadOnly );
			ds >> variant;
			switch ( variant.type() )
			{
			case QVariant::String:
				emit receivedText( variant.toString() );
				break;
			case QVariant::Image:
				emit receivedPixmap( QPixmap(variant.toImage()) );
				break;
			case QVariant::Pixmap:
				emit receivedPixmap( variant.toPixmap() );
				break;
			default:
				break;
			}
		}
	}
}
