
#include "kwhiteboard_part.h"
#include "kwbview.h"
#include "server.h"
#include "client.h"

#include <kinstance.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kfiledialog.h>
#include <knuminput.h> 

#include <qfile.h>
#include <qtextstream.h>
#include <qmultilineedit.h>
#include <qobject.h>
#include <qlabel.h>
#include <qstringlist.h>

kwhiteboardPart::kwhiteboardPart( QWidget *parentWidget, const char *widgetName,
                                  QObject *parent, const char *name, const QStringList &args)
		: KParts::ReadWritePart(parent, name)
{
	// we need an instance
	setInstance( kwhiteboardPartFactory::instance() );

	// this should be your custom internal widget
	m_widget = new KWBView( parentWidget, widgetName );
	
	connect(m_widget,SIGNAL(connectToServer()),this,SLOT(connectToServer()));

	m_client = new KWBClient();
	connect( m_widget, SIGNAL(sendText(const QString&)), m_client, SLOT(sendText(const QString&)) );
	connect( m_widget, SIGNAL(sendImage()), m_client, SLOT(sendImage()) );
	connect( m_widget->getWBWidget(),SIGNAL(sendBuffer(const QPixmap&)),m_client, SLOT(sendPixmap(const QPixmap& )));

	connect(m_client,SIGNAL(receivedPixmap(const QPixmap& )),m_widget->getWBWidget(),SLOT(applyBuffer(const QPixmap& )));
	connect(m_client,SIGNAL(receivedText(const QString&)),m_widget->getTextEdit(),SLOT(append(const QString& )));
	
	
	m_server = new KWBServer(args[0].toInt(),this);
	connect(m_server,SIGNAL(signalNewConnection(const int& )),m_client,SLOT(setSocket(const int& )));
	
	// notify the part that this is our internal widget
	setWidget(m_widget);

	// create our actions
	KStdAction::open(this, SLOT(fileOpen()), actionCollection());
	KStdAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
	KStdAction::save(this, SLOT(save()), actionCollection());

	// set our XML-UI resource file
	setXMLFile("kwhiteboard_part.rc");

	// we are read-write by default
	setReadWrite(true);

	// we are not modified since we haven't done anything yet
	setModified(false);
}

void kwhiteboardPart::connectToServer()
{
	m_socket = new QSocket( this );
	m_socket->connectToHost( "localhost", m_widget->getPortNumEdit()->value() );
	m_client->setSocket(m_socket->socket());
}


kwhiteboardPart::~kwhiteboardPart()
{}

void kwhiteboardPart::setReadWrite(bool rw)
{
	/*
	// notify your internal widget of the read-write state
	m_widget->setReadOnly(!rw);
	if (rw)
		connect(m_widget, SIGNAL(textChanged()),
		        this,     SLOT(setModified()));
	else
	{
		disconnect(m_widget, SIGNAL(textChanged()),
		           this,     SLOT(setModified()));
	}
	*/
	ReadWritePart::setReadWrite(rw);
}

void kwhiteboardPart::setModified(bool modified)
{
	// get a handle on our Save action and make sure it is valid
	KAction *save = actionCollection()->action(KStdAction::stdName(KStdAction::Save));
	if (!save)
		return;

	// if so, we either enable or disable it based on the current
	// state
	if (modified)
		save->setEnabled(true);
	else
		save->setEnabled(false);

	// in any event, we want our parent to do it's thing
	ReadWritePart::setModified(modified);
}

bool kwhiteboardPart::openFile()
{
	/*
	// m_file is always local so we can use QFile on it
	QFile file(m_file);
	if (file.open(IO_ReadOnly) == false)
		return false;

	// our example widget is text-based, so we use QTextStream instead
	// of a raw QDataStream
	QTextStream stream(&file);
	QString str;
	while (!stream.eof())
		str += stream.readLine() + "\n";

	file.close();

	// now that we have the entire file, display it
	m_widget->setText(str);

	// just for fun, set the status bar
	emit setStatusBarText( m_url.prettyURL() );
	*/
	return true;
}

bool kwhiteboardPart::saveFile()
{
	/*
	// if we aren't read-write, return immediately
	if (isReadWrite() == false)
		return false;

	// m_file is always local, so we use QFile
	QFile file(m_file);
	if (file.open(IO_WriteOnly) == false)
		return false;

	// use QTextStream to dump the text to the file
	QTextStream stream(&file);
	stream << m_widget->text();

	file.close();
	*/
	return true;
}

void kwhiteboardPart::fileOpen()
{
	/*
	// this slot is called whenever the File->Open menu is selected,
	// the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
	// button is clicked
	QString file_name = KFileDialog::getOpenFileName();

	if (file_name.isEmpty() == false)
		openURL(file_name);
	*/
}

void kwhiteboardPart::fileSaveAs()
{
	/*
	// this slot is called whenever the File->Save As menu is selected,
	QString file_name = KFileDialog::getSaveFileName();
	if (file_name.isEmpty() == false)
		saveAs(file_name);
	*/
}


// It's usually safe to leave the factory code alone.. with the
// notable exception of the KAboutData data
#include <kaboutdata.h>
#include <klocale.h>

KInstance*  kwhiteboardPartFactory::s_instance = 0L;
KAboutData* kwhiteboardPartFactory::s_about = 0L;

kwhiteboardPartFactory::kwhiteboardPartFactory()
		: KParts::Factory()
{}

kwhiteboardPartFactory::~kwhiteboardPartFactory()
{
	delete s_instance;
	delete s_about;

	s_instance = 0L;
}

KParts::Part* kwhiteboardPartFactory::createPartObject( QWidget *parentWidget, const char *widgetName,
        QObject *parent, const char *name,
        const char *classname, const QStringList &args )
{
	// Create an instance of our Part
	kwhiteboardPart* obj = new kwhiteboardPart( parentWidget, widgetName, parent, name,args );

	// See if we are to be read-write or not
	if (QCString(classname) == "KParts::ReadOnlyPart")
		obj->setReadWrite(false);

	return obj;
}

KInstance* kwhiteboardPartFactory::instance()
{
	if( !s_instance )
	{
		s_about = new KAboutData("kwhiteboardpart", I18N_NOOP("kwhiteboardPart"), "0.1");
		s_about->addAuthor("Malte Bhme", 0, "malte@stammkranich.de");
		s_instance = new KInstance(s_about);
	}
	return s_instance;
}

extern "C"
{
	void* init_libkwhiteboardpart()
	{
		return new kwhiteboardPartFactory;
	}
};

#include "kwhiteboard_part.moc"
