
#ifndef _KWHITEBOARDPART_H_
#define _KWHITEBOARDPART_H_

#include <kparts/part.h>
#include <kparts/factory.h>

class QWidget;
class QPainter;
class KURL;
class KWBView;
class KWBServer;
class KWBClient;
class QSocket;
class QStringList;

/**
 * This is a "Part".  It that does all the real work in a KPart
 * application.
 *
 * @short Main Part
 * @author Malte Bhme <malte@stammkranich.de>
 * @version 0.1
 */
class kwhiteboardPart : public KParts::ReadWritePart
{
    Q_OBJECT
public:
    /**
     * Default constructor
     */
    kwhiteboardPart(QWidget *parentWidget, const char *widgetName,
                    QObject *parent, const char *name, const QStringList &args);

    /**
     * Destructor
     */
    virtual ~kwhiteboardPart();

    /**
     * This is a virtual function inherited from KParts::ReadWritePart.  
     * A shell will use this to inform this Part if it should act
     * read-only
     */
    virtual void setReadWrite(bool rw);

    /**
     * Reimplemented to disable and enable Save action
     */
    virtual void setModified(bool modified);

protected:
    /**
     * This must be implemented by each part
     */
    virtual bool openFile();

    /**
     * This must be implemented by each read-write part
     */
    virtual bool saveFile();

protected slots:
    void fileOpen();
    void fileSaveAs();
	void connectToServer();
	
private:
    KWBView *m_widget;
	KWBServer *m_server;
	KWBClient *m_client;
	QSocket *m_socket;	
};

class KInstance;
class KAboutData;

class kwhiteboardPartFactory : public KParts::Factory
{
    Q_OBJECT
public:
    kwhiteboardPartFactory();
    virtual ~kwhiteboardPartFactory();
    virtual KParts::Part* createPartObject( QWidget *parentWidget, const char *widgetName,
                                            QObject *parent, const char *name,
                                            const char *classname, const QStringList &args );
    static KInstance* instance();
 
private:
    static KInstance* s_instance;
    static KAboutData* s_about;
};

#endif // _KWHITEBOARDPART_H_
