/***************************************************************************
 *   Copyright (C) 2004 by Malte Bhme                                     *
 *   malte@stammkranich.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qapplication.h>
#include <qevent.h>
#include <qpainter.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qspinbox.h>
#include <qtooltip.h>
#include <qrect.h>
#include <qpoint.h>
#include <qcolordialog.h>
#include <qfiledialog.h>
#include <qcursor.h>
#include <qimage.h>
#include <qstrlist.h>
#include <qpopupmenu.h>
#include <qintdict.h>


#include "widget.h"

const bool no_writing = FALSE;

KWBWidget::KWBWidget( QWidget *parent, const char *name )
		: QWidget( parent, name, WStaticContents ), pen( Qt::red, 3 ), polyline(3),
		mousePressed( FALSE ), buffer( width(), height() )
{

	buffer.fill( colorGroup().base() );
	setBackgroundMode( QWidget::PaletteBase );

	setCursor( Qt::crossCursor );

}

void KWBWidget::save( const QString &filename, const QString &format )
{
	if ( !no_writing )
		buffer.save( filename, format.upper() );
}

void KWBWidget::clearScreen()
{
	buffer.fill( colorGroup().base() );
	repaint( FALSE );
}

void KWBWidget::applyBuffer(const QPixmap &newBuffer)
{
	buffer = newBuffer;
	repaint( FALSE );
}

void KWBWidget::mousePressEvent( QMouseEvent *e )
{
	mousePressed = TRUE;
	polyline[2] = polyline[1] = polyline[0] = e->pos();
}

void KWBWidget::mouseReleaseEvent( QMouseEvent * )
{
	mousePressed = FALSE;
	emit sendBuffer(buffer);
}

void KWBWidget::mouseMoveEvent( QMouseEvent *e )
{
	if ( mousePressed )
	{
		QPainter painter;
		painter.begin( &buffer );
		painter.setPen( pen );
		polyline[2] = polyline[1];
		polyline[1] = polyline[0];
		polyline[0] = e->pos();
		painter.drawPolyline( polyline );
		painter.end();

		QRect r = polyline.boundingRect();
		r = r.normalize();
		r.setLeft( r.left() - penWidth() );
		r.setTop( r.top() - penWidth() );
		r.setRight( r.right() + penWidth() );
		r.setBottom( r.bottom() + penWidth() );

		//bitBlt( this, r.x(), r.y(), &buffer, r.x(), r.y(), r.width(), r.height() );
		//bitBlt( &buffer, r.x(), r.y(), &buffer2, r.x(), r.y(), r.width(), r.height() );
		repaint(FALSE);
	}
}

void KWBWidget::resizeEvent( QResizeEvent *e )
{
	QWidget::resizeEvent( e );

	int w = width() > buffer.width() ?
	        width() : buffer.width();
	int h = height() > buffer.height() ?
	        height() : buffer.height();

	QPixmap tmp( buffer );
	buffer.resize( w, h );
	buffer.fill( colorGroup().base() );
	bitBlt( &buffer, 0, 0, &tmp, 0, 0, tmp.width(), tmp.height() );
}

void KWBWidget::paintEvent( QPaintEvent *e )
{
	QWidget::paintEvent( e );

	QMemArray<QRect> rects = e->region().rects();
	for ( uint i = 0; i < rects.count(); i++ )
	{
		QRect r = rects[(int)i];
		bitBlt( this, r.x(), r.y(), &buffer, r.x(), r.y(), r.width(), r.height() );
	}
}


KWBWidget::~KWBWidget()
{}


#include "widget.moc"

